#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
"""
Fetch plans exist to manage running each of a group of FileFetchers. At the moment,
we have identified only two kinds of fetch plan:

 - Sequential fetching, where we do one fetcher and then the next until all are complete
 - Parallel fetching, where we allocate a thread pool of a certain size and send all of
   the fetchers to the pool to be run as threads become free

The size of the thread pool is the concurrency number. A ParallelFetchPlan with
concurrency = 7 will generate 7 threads and fetch up to 7 files at the same time.
This means if you had (say) N > 7 files to fetch, you would always have 7 of them
active until you reach the last 7, and then they would wind down to 0 as they
complete.

It turns out that sequential fetching is a degenerate case of parallel fetching, where
the concurrency limit is 1. So there is no need to implement a sequential fetcher, even
though it would be a quite trivial for loop. We can simply create a thread pool with
one worker and put all the tasks in that pool.
"""

import concurrent.futures
from typing import List, Union

from .interfaces import FetchPlan, FileFetcher, FetchContext


# pylint: disable=E0401, E0402, E1136, R0903


class ParallelFetchPlan(FetchPlan):
    """
    Fetch a number of sub-fetchers or sub-plans. Executes as many
    concurrently as permitted by the concurrency limit.
    """

    def __init__(self, fetchers: List[Union[FileFetcher, FetchPlan]], concurrency: int = 1):
        self.fetchers = fetchers
        self.concurrency = concurrency

    def fetch(self, context: FetchContext):
        """
        Carry out the fetch plan.

        :return:
        """
        # treat the list of fetchers as a "work log" and work down it
        with concurrent.futures.ThreadPoolExecutor(max_workers=self.concurrency) as executor:
            # submit all the fetchers to the thread pool to be executed
            futures = [executor.submit(fetcher.fetch, context) for fetcher in self.fetchers]

            # now we can wait for them all to complete
            for future in concurrent.futures.as_completed(futures):
                # under normal operation, we won't care about the result of the
                # fetch operation for these futures, only whether or not there
                # is an exception
                future.result()
